#-*- encoding: utf-8 -*-
#-------------------------------------------------------------------------------
# Name:        palette.py
# Purpose:
#
# Author:      Eiichi Takebuchi(GRGSIBERIA)
#
# Created:     07/05/2011
# Copyright:   (c) Eiichi Takebuchi(GRGSIBERIA) 2011
# Licence:     <your licence>
#-------------------------------------------------------------------------------
#!/usr/bin/env python
from palette import *
from png import *
import sys

class NESImage:
    def CreatePalette(self, in_pal, out_pal):
        obj = EdgePalette()
        pal = obj.loadEdgePalette32(in_pal)
        col = obj.getPaletteColors()

        result = []
        for p in pal:
            c_ind = 0
            for c in col:
                cnt = 0
                for i in range(3):
                    cnt += 1 if p[i] == c[i] else 0
                if cnt == 3:    # 3色一致なら登録して次の色へ
                    result.append(c_ind)
                    break
                c_ind += 1

        f = open(out_pal, "wb")
        for o in result:
            f.write(pack("!B", o))
        f.close()

    def CreateEdgePalette(self, out_pal):
        obj = EdgePalette()
        obj.createEdgePalette64(out_pal)

    def __ExtractUnitFromColor(self, ind):
        result = []
        for i in range(16):
            unit = []
            for j in range(16):
                spr = []
                for h in range(8):
                    for w in range(8):
                        a = i * 8 + h
                        b = j * 8 + w
                        spr.append(ind[a][b])
                unit.append(spr)
            result.append(unit)
        return result

    # 0～3までのビットパターン生成
    def __CreateBitPettern(self, spr):
        for pix in spr:
            pix %= 4

    # ビットパターンを分離
    def __SplitBitPettern(self, spr):
        x = []
        y = []
        for pix in spr:
            y.append((pix & 0b00000010) >> 1)
            x.append(pix & 0b00000001)
        return x, y

    # 1byteに1列を入れていく
    def __ComposeBits(self, xx, yy):
        cnt = 7
        x = 0
        y = 0
        rx = []
        ry = []
        for i in range(64):
            x += xx[i] << cnt
            y += yy[i] << cnt
            cnt -= 1
            if cnt < 0:
                cnt = 7
                rx.append(x)
                ry.append(y)
                x = 0
                y = 0
        return rx, ry

    def __BurnBits(self, f, xx, yy):
        for x in xx:
            f.write(pack("!B", x))
        for y in yy:
            f.write(pack("!B", y))

    def __WriteNESImage(self, out_name, ind):
        f = open(out_name, "wb")
        for unit in ind:
            for spr in unit:
                self.__CreateBitPettern(spr)
                x, y = self.__SplitBitPettern(spr)
                self.__ComposeBits(x, y)
                x, y = self.__ComposeBits(x, y)
                self.__BurnBits(f, x, y)
        f.close()

    def CreateNESImage(self, in_file, out_name):
        obj = EdgePng(in_file)
        ind = obj.getColorIndices()
        result = self.__ExtractUnitFromColor(ind)
        self.__WriteNESImage(out_name, result)

def PrintHelp():
    print
    print u":*:・'゜☆' .:*:・'゜☆' .:*:・'゜☆' .:*:・'゜☆' .:*:・☆"
    print u"         NESImageExtracter form PNG"
    print u"             (c) Eiichi Takebuchi(GRGSIBERIA) 2011"
    print u":*:・'゜☆' .:*:・'゜☆' .:*:・'゜☆' .:*:・'゜☆' .:*:・☆"
    print
    print u"-h           help"
    print u"-i in out    PNGファイルからNES形式の画像を出力"
    print u"-p in out    EDGE形式のパレットからNES形式のパレットを出力"
    print u"-e out       EDGE形式の空のパレットを出力"

def PrintError():
    print "引数が足りません"

def CommandImage(argv, obj):
    if len(argv) != 3:
        PrintError()
    else:
        obj.CreateNESImage(argv[1], argv[2])

def CommandPalette(argv, obj):
    if len(argv) != 3:
        PrintError()
    else:
        obj.CreatePalette(argv[1], argv[2])

def CommandVoid(argv, obj):
    if len(argv) != 2:
        PrintError()
    else:
        obj.CreateEdgePalette(argv[1])

def main():
    argvs = sys.argv
    del argvs[0]
    argc = len(argvs)
    if argc == 0 or argvs[0] == "-h":
        PrintHelp()
    else:
        obj = NESImage()
        if argvs[0] == "-i":
            CommandImage(argvs, obj)
        elif argvs[0] == "-p":
            CommandPalette(argvs, obj)
        elif argvs[0] == "-e":
            CommandVoid(argvs, obj)

if __name__ == "__main__":
    main()