#-*- encoding: utf-8 -*-
#-------------------------------------------------------------------------------
# Name:        png.py
# Purpose:
#
# Author:      Eiichi Takebuchi(GRGSIBERIA)
#
# Created:     06/05/2011
# Copyright:   (c) Eiichi Takebuchi(GRGSIBERIA) 2011
# Licence:     <your licence>
#-------------------------------------------------------------------------------
#!/usr/bin/env python
from struct import *
import zlib

class DoNotPalettePngException(BaseException):
    def printException(self):
        print "パレットを使うタイプのPNGじゃないよ"

class PropertyException(BaseException):
    def __init__(self, s):
        self.__s = s

    def printException(self):
        print "アクセスできません: " + self.__s

class ImageSizeException(BaseException):
    def __init__(self, w, h):
        self.__w = w
        self.__h = h

    def printException(self):
        print "画像のサイズが不適切です"
        print "WIDTH :" + self.__w
        print "HEIGHT:" + self.__h

class EdgePng:
    @property
    def width(self):
        return self.__width

    @width.setter
    def width(self, val):
        raise PropertyException("EdgePng.width.setter")

    @width.deleter
    def width(self):
        raise PropertyException("EdgePng.width.deleter")

    @property
    def height(self):
        return self.__height

    @height.setter
    def height(self, val):
        raise PropertyException("EdgePng.height.setter")

    @height.deleter
    def height(self):
        raise PropertyException("EdgePng.height.deleter")

    @property
    def plte_size(self):
        return self.__plte_size

    @plte_size.setter
    def plte_size(self, val):
        raise PropertyException("EdgePng.plte_size.setter")

    @plte_size.deleter
    def plte_size(self):
        raise PropertyException("EdgePng.plte_size.deleter")

    @property
    def idat_size(self):
        return self.__idat_size

    @idat_size.setter
    def idat_size(self, val):
        raise PropertyException("EdgePng.idat_size.setter")

    @idat_size.deleter
    def idat_size(self):
        raise PropertyException("EdgePng.idat_size.deleter")

    def __getIHDR(self):
        f = open(self.__fname, "rb")
        self.__searchstr(f, "IHDR")
        self.__width = unpack("!L", f.read(4))[0]
        self.__height = unpack("!L", f.read(4))[0]
        if not (self.__width == 128 and self.__height == 128):
             raise ImageSizeException(self.__width, self.__height)
        f.seek(1,1)
        if unpack("!B", f.read(1))[0] != 3: raise DoNotPalettePngException()
        f.close()

    def __init__(self, filename):
        self.__fname = filename
        self.__getIHDR()

    def getPalette(self):
        f = open(self.__fname, "rb")
        self.__searchstr(f, "PLTE")
        f.seek(-8, 1)
        size = unpack("!L", f.read(4))[0] / 3
        self.__plte_size = size * 3
        f.seek(4, 1)

        result = []
        for i in range(size):
            buf = []
            for j in range(3):
                buf.append(unpack("!B", f.read(1))[0])
            result.append(buf)

        f.close()
        return result

    # 色番号だけ取得
    def getColorIndices(self):
        f = open(self.__fname, "rb")
        self.__searchstr(f, "IDAT")
        f.seek(-8, 1)
        size = unpack("!L", f.read(4))[0]
        self.__idat_size = size
        f.seek(4, 1)

        result = []
        l = self.__width+1
        strbuf = zlib.decompress(f.read(size))
        for h in range(self.__height):
            buf = []
            for w in range(l):
                if w == 0: continue
                buf.append(unpack("!B", strbuf[l*h+w])[0])
            result.append(buf)

        f.close()
        return result

    def __searchstr(self, fp, s):
        length = len(s)
        itr = 0
        buf = []

        # 最初に文字数だけ読み込み
        for i in range(length):
            itr = fp.read(1)
            buf.append(itr)

        while True:
            # 比較用文字列作って合致していたら終了
            string = ""
            for i in buf:
                string += i
            if string == s:
                return True

            # 最後まで行ったら失敗
            itr = fp.read(1)
            if not itr: return True

            buf.append(itr)
            del buf[0]
        return False